package edu.upvehu.gbg.ejercicioclase;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

/*
 * NOTA.- La documentación es importante. Por ser un ejemplo, y el tiempo limitado, no la incluimos.
 */

/**
 * @author clase
 */
class DBMummec2324 implements AutoCloseable {

    static DBMummec2324 elSingleton = null;
    static Connection con;
    private static final String DATABASE_NAME = "MUMMEC2324";

    static DBMummec2324 newInstance() throws SQLException {
        if (elSingleton == null) {
            elSingleton = new DBMummec2324();
        }
        return elSingleton;
    }

    // Implementación del interface Closeable
    @Override
    public void close() throws IOException {
        try {
            if (con != null) {
                con.close();
            }
        } catch (SQLException ex) {
            throw new IOException("No puedo cerrar la conexion con la base de datos", ex);
        }
    }

    //Re-construcción del esquema de la base de datos.
    void instanciaEsquema() throws SQLException {
        con.createStatement().executeUpdate("DROP DATABASE IF EXISTS " + DATABASE_NAME + ";");
        con.createStatement().executeUpdate("CREATE DATABASE " + DATABASE_NAME + ";");
        con.createStatement().executeUpdate("USE " + DATABASE_NAME + ";");

        con.setAutoCommit(false);
        con.createStatement().executeUpdate("CREATE TABLE papersreferences ( ref_paper INTEGER, cited_ref_paper INTEGER, PRIMARY KEY(ref_paper,cited_ref_paper) );");
        con.createStatement().executeUpdate("CREATE TABLE publication ( ref_publication INTEGER PRIMARY KEY, title VARCHAR(200) );");
        con.createStatement().executeUpdate("CREATE TABLE papersauthors ( ref_paper INTEGER, ref_author INTEGER, PRIMARY KEY(ref_paper,ref_author) );");
        con.createStatement().executeUpdate("CREATE TABLE authorafiliation ( ref_author INTEGER, ref_afiliation INTEGER, PRIMARY KEY(ref_author, ref_afiliation) );");
        con.createStatement().executeUpdate("CREATE TABLE afiliations ( ref_afiliation INTEGER PRIMARY KEY, university VARCHAR(65), webpage VARCHAR(30), country VARCHAR(15) );");
        con.createStatement().executeUpdate("CREATE TABLE papersinfo ( ref_paper INTEGER PRIMARY KEY, title VARCHAR(200), publiyear INTEGER, ref_publication INTEGER, url VARCHAR(200));");
        con.createStatement().executeUpdate("CREATE TABLE authors ( ref_author INTEGER PRIMARY KEY, name VARCHAR(25) );");
        con.commit();
    }
    
    //PARA autores y publicaciones
    void arrayOfStringsToDB(String[] valores, String tableName) throws SQLException {
        PreparedStatement ps = con.prepareStatement("insert into " + tableName + " values (?,?);");
        int i = 0;
        for (String s : valores) {
            ps.setInt(1, ++i);
            ps.setString(2, s);
            ps.addBatch();
        }
        flush(ps);
    }

    //PARA papersauthors y papersreferences
    void listOf2IntsToDB(List<int[]> autfil, String table) throws SQLException {
        PreparedStatement ps = con.prepareStatement("insert into " + table + " values (?,?);");
        for (int[] par : autfil) {
            ps.setInt(1, par[0]);
            ps.setInt(2, par[1]);
            ps.addBatch();
        }
        flush(ps);
    }
    
    void filiacionesToDB(String[] valores) throws SQLException {
        PreparedStatement ps = con.prepareStatement("insert into afiliations values (?,?,?,?);");
        int i = 0;
        for (String s : valores) {
            String[] campos = s.split("\\s*,\\s*");
            ps.setInt(1, ++i);
            ps.setString(2, campos[0]);
            ps.setString(3, campos[1]);
            ps.setString(4, campos[2]);
            ps.addBatch();
        }
        flush(ps);
    }

    void infoToDB(List<String[]> info) throws SQLException {
        PreparedStatement ps = con.prepareStatement("insert into papersinfo values (?,?,?,?,?);");
        for (String[] campos : info) {
            ps.setInt(1, Integer.parseInt(campos[0]));  //Numero de referencia
            ps.setString(2, campos[1]);                 //Titulo
            ps.setInt(3, Integer.parseInt(campos[2]));  //Año
            ps.setInt(4, Integer.parseInt(campos[3]));  //Publicacion (indice)
            ps.setString(5, campos[4]);                 //Link
            ps.addBatch();
        }
        flush(ps);
    }

    //ZONA PRIVADA
    //El constructor (privado para ser Singleton)
    private DBMummec2324() throws SQLException {
        con = DriverManager.getConnection("jdbc:mysql://localhost:3306/?serverTimezone=UTC", "USUARIO", "CLAVE");
    }

    private void flush(PreparedStatement ps) throws SQLException {
        ps.executeBatch();
        con.commit();
    }

}
