package edu.upvehu.gbg.ejercicioclase;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.swing.JOptionPane;

/*
 * NOTA.- La documentación es importante. Por ser un ejemplo, y el tiempo limitado, no la incluimos.
 * 
 * NOTA 2.- He eliminado la excepción que se lanzaba en el main(.). Para ello genero el objeto URL directamente 
 * dentro del constructor de InputStreamReader y la excepción es recogida en el catch final. No aparece
 * explícitamente porque MalformedURLException es hija de IOException y por tanto ya está incluida.
 * 
 * NOTA 3.- He unido en un "catch" los tres que teníamos, cosa que se puede hacer cuando vamos a aplicar la misma
 * acción a todos. Como acción que retirado la que pone Netbeans (usar un objeto que genera "logs") y he puesto 
 * una presentación de un diálogo gráfico.
 */

/**
 * @author clase
 */
public class Solucion  {
    
    private static final String URL="https://gtts.ehu.es/German/Docencia/assets/DB.txt";
    

    public static void main(String[] args) {
        try (BufferedReader  br=new BufferedReader(new InputStreamReader(new URL(URL).openStream(),"utf8"));
                DBMummec2324 db=DBMummec2324.newInstance();) {
            String line;
            while((line=br.readLine())!=null) Referencia.addLine(line);
            
          
          //(Re)generamos la base de datos
            db.instanciaEsquema();
            
            //Generamos la tabla de publicaiones, que es un caso simple (un array de strings)
            String[] publicaciones=Referencia.getAllPublicaciones().toArray(new String[0]);
            db.arrayOfStringsToDB(publicaciones, "publication");            
            
            //VAMOS A RESOLVER TODO LO DE AUTORES Y FILIACIONES
            //Pedimos todos los campos de autor con su filiación
            Set<String> authsFilis=Referencia.getAllAutoresFiliaciones();
            //Y generamos sets ordenados con autores y filiaciones
            // - Sets ordenados porque hay repeticiones y queremos eliminarlas
            // - Ordenados porque pasarán a arrays donde haremos busquedas binarias posteriormente
            SortedSet<String> auths=new TreeSet(), filis=new TreeSet();
            for (String s:authsFilis){
                String[] campos=s.trim().replaceFirst("^(.*)\"(.*)\"$", "$1<SEP>$2").split("<SEP>");
                auths.add(campos[0].trim());
                filis.add(campos[1].trim());
            }
            //Los pasamos a array por conveniencia para generar buscar índices de autores
            String[] autores=auths.toArray(new String[0]);
            String[] filiaciones=filis.toArray(new String[0]);
            //Esto ya puede ir a la base de datos
            db.arrayOfStringsToDB(autores, "authors");
            db.filiacionesToDB(filiaciones);   
            
            //Y ahora generaremos el mapa que relaciona cada autor con su filiacion como indices de los arrays
            List<int[]> autfil= new ArrayList<>();
            for (String aut:autores) {
                String fil=null;
                for (String s:authsFilis) if (s.startsWith(aut)) {fil=s.trim().replaceFirst("^.*\"(.*)\"$", "$1").trim(); break;}
                autfil.add(new int[]{1+Arrays.binarySearch(autores, aut), 1+Arrays.binarySearch(filiaciones, fil)});
            }
            db.listOf2IntsToDB(autfil,"authorafiliation");            
            
            //Generamos la tabla de autor-filiación recorriendo todas las referencias    
            List<int[]> paperAuth= new ArrayList<>();
            for (Referencia v : Referencia.getAllRefs()){
                for (String af:v.getAutores()){
                    String aut=af.replaceFirst("(.*?)\".*$", "$1").trim();
                    paperAuth.add(new int[]{Integer.parseInt(v.getRefNum()), 1+Arrays.binarySearch(autores, aut)});
                }
            }    
            db.listOf2IntsToDB(paperAuth,"papersauthors");            

            //Generamos la tabla de referencias cruzadas recorriendo todas las referencias    
            List<int[]> paperRefs= new ArrayList<>();
            for (Referencia v : Referencia.getAllRefs()){
                String referencias=v.getReferencias();
                if (referencias!=null)
                    for (String ref:referencias.split("\\s+"))
                        paperRefs.add(new int[]{Integer.parseInt(v.getRefNum()), Integer.parseInt(ref)});
            }    
            db.listOf2IntsToDB(paperRefs,"papersreferences");            
            
            //Generamos la tabla de  info recorriendo todas las referencias    
            List<String[]> strings= new ArrayList<>();
            for (Referencia v : Referencia.getAllRefs())
                strings.add(new String[]{v.getRefNum(), v.getTitulo(), v.getAnio(),""+(1+Arrays.binarySearch(publicaciones,v.getPublicacion())), v.getLink()});
            db.infoToDB(strings);  

        } catch (IOException | Referencia.IllegalFieldException | SQLException ex){
            JOptionPane.showMessageDialog(null, "<html>Se ha producido un error:<br>"+ex);
        }   
    }
}
